library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity datapath is	 
  port (			  
	  clk, reset : in std_logic;
    in1, in2, in3, in4 : in std_logic_vector(7 downto 0);
		sel : in std_logic_vector(2 downto 0);
		outp : out std_logic_vector(7 downto 0)
  );
end datapath;

architecture structural of datapath is
  component cplxfunc is	 
    port (			  
      a, b, c, d : in std_logic_vector(7 downto 0);
		  o1, o2, o3 : out std_logic_vector(7 downto 0)
    );
  end component;
	component regn is	 
    generic (
      N : integer := 16
    );
    port (			  
      clk   : in std_logic;
		  reset : in std_logic;
      d     : in std_logic_vector(N-1 downto 0);
      q     : out std_logic_vector(N-1 downto 0)
    );
  end component;
	--
  signal x1, x2, x3, x4, x4_r: std_logic_vector(7 downto 0);
begin                              

  uut1 : cplxfunc 
	  port map (
	    a => in1, b => in2, c => in3, d => in4,
		  o1 => x1, o2 => x2, o3 => x3
    );
	
  G_mux1 : for i in 7 downto 0 generate
	  with sel(1 downto 0) select
		  x4(i) <= x3(i) when "10",
			         x2(i) when "01",
							 x1(i) when others;
  end generate G_mux1;
	
  uut3 : regn 
		generic map (
		  N => 8
		)
		port map (
		  clk   => clk,
		  reset => reset,
      d     => x4,
		  q     => x4_r
    );
  
  G_mux2 : for i in 7 downto 0 generate
	  with sel(2 downto 2) select
		  outp(i) <= x4_r(i) when "1",
			         x4(i) when others;
  end generate G_mux2;

end structural;
