-- 
-- Modelsim and GHDL example of top-level file for a datapath
--
-- Graph representation:
-- 
--       vr1 vr2        vr3 vr4
--       |   |          |   |
--        AND            ADD
--         | _vr5         | vr6
--         |  -_______    |
--        MOV         -- XOR
--         |              |
--         vr7            vr8
--
-- NOTES
-- 1. Allaja to paradeigma gia na balw kai mia XOR.
-- 2. MAT8AIE: sto paradeigma mou den xrhsimopoiw KATAXWRHTES gia tis endiameses metablhtes.
-- Sto paradeigma pou sou stelnw, exw kataxwrhtes mono gia ta:
-- vr1, vr2, vr3, vr4, vr7, vr8. Kanonika, epilegoume basei kapoiwn krithriwn (epi8ymhtos 
-- kyklos mhxanhs se ns apo th syn8esh) an prepei na mpoun h oxi kai pou orismenoi 
-- kataxwrhtes.

library IEEE;
use IEEE.std_logic_1164.all;

entity datapath is
  generic (
	  DW     : integer := 32
  );  
	port (
	  clk    : in  std_logic;
    reset  : in  std_logic;
		vr1    : in  std_logic_vector(DW-1 downto 0);
		vr2    : in  std_logic_vector(DW-1 downto 0);
		vr3    : in  std_logic_vector(DW-1 downto 0);
		vr4    : in  std_logic_vector(DW-1 downto 0);
		vr7    : out std_logic_vector(DW-1 downto 0);
		vr8    : out std_logic_vector(DW-1 downto 0);
		vr1_en : in  std_logic;
		vr2_en : in  std_logic;
		vr3_en : in  std_logic;
		vr4_en : in  std_logic;
		vr7_en : in  std_logic;
		vr8_en : in  std_logic
  );
end datapath;


architecture struct of datapath is
  -- Component declarations
  component instr_and
    generic (
      DW  : integer := 32
    );
    port (
		  a_i : in  std_logic_vector(DW-1 downto 0);
      b_i : in  std_logic_vector(DW-1 downto 0);
      c_o : out std_logic_vector(DW-1 downto 0)  
    );
  end component;
  --
  component instr_xor
    generic (
      DW  : integer := 32
    );
    port (
		  a_i : in  std_logic_vector(DW-1 downto 0);
      b_i : in  std_logic_vector(DW-1 downto 0);
      c_o : out std_logic_vector(DW-1 downto 0)  
    );
  end component;
  --
  component instr_add
    generic (
      DW  : integer := 32
    );
    port (
		  a_i : in  std_logic_vector(DW-1 downto 0);
      b_i : in  std_logic_vector(DW-1 downto 0);
      c_o : out std_logic_vector(DW-1 downto 0)  
    );
  end component;
  --
  component instr_mov
    generic (
      DW  : integer := 32
    );
    port (
		  a_i : in  std_logic_vector(DW-1 downto 0); 
			c_o : out std_logic_vector(DW-1 downto 0)  
    );
  end component;
  --
  component reg_dw
    generic (
      DW : integer := 32
    );
      port (
			  clk   : in  std_logic;
        reset : in  std_logic;
        load  : in  std_logic;
        d     : in  std_logic_vector(DW-1 downto 0);
        q     : out std_logic_vector(DW-1 downto 0)
      );
    end component;
  --
	-- Signal declarations
	signal vr1_reg      : std_logic_vector(DW-1 downto 0);
	signal vr2_reg      : std_logic_vector(DW-1 downto 0);
	signal vr3_reg      : std_logic_vector(DW-1 downto 0);
	signal vr4_reg      : std_logic_vector(DW-1 downto 0);
	signal vr5          : std_logic_vector(DW-1 downto 0);
  signal vr6          : std_logic_vector(DW-1 downto 0);
	signal vr7_internal : std_logic_vector(DW-1 downto 0);
	signal vr8_internal : std_logic_vector(DW-1 downto 0);
	signal vr7_reg      : std_logic_vector(DW-1 downto 0);
	signal vr8_reg      : std_logic_vector(DW-1 downto 0);
  --
begin

  and_1 : instr_and 
	  generic map ( 
		  DW => DW
		) 
		port map ( 
		  a_i => vr1_reg,
		  b_i => vr2_reg,
		  c_o => vr5
		);
		
  add_1 : instr_add 
	  generic map ( 
		  DW => DW
		) 
		port map ( 
		  a_i => vr3_reg,
		  b_i => vr4_reg,
		  c_o => vr6
		);

  mov_1 : instr_mov
	  generic map ( 
		  DW => DW
		) 
		port map ( 
		  a_i => vr5,
		  c_o => vr7_internal
		);
		
  xor_1 : instr_xor 
	  generic map ( 
		  DW => DW
		) 
		port map ( 
		  a_i => vr5,
		  b_i => vr6,
		  c_o => vr8_internal
		);

  r1 : reg_dw 
	  generic map (
		  DW => DW
    )
		port map (
		  clk   => clk,
			reset => reset,
			load  => vr1_en,
			d     => vr1, 
			q     => vr1_reg
    );
		
  r2 : reg_dw 
	  generic map (
		  DW => DW
    )
		port map (
		  clk   => clk,
			reset => reset,
			load  => vr2_en,
			d     => vr2, 
			q     => vr2_reg
    );

  r3 : reg_dw 
	  generic map (
		  DW => DW
    )
		port map (
		  clk   => clk,
			reset => reset,
			load  => vr3_en,
			d     => vr3, 
			q     => vr3_reg
    );

  r4 : reg_dw 
	  generic map (
		  DW => DW
    )
		port map (
		  clk   => clk,
			reset => reset,
			load  => vr4_en,
			d     => vr4, 
			q     => vr4_reg
    );

  r7 : reg_dw 
	  generic map (
		  DW => DW
    )
		port map (
		  clk   => clk,
			reset => reset,
			load  => vr7_en,
			d     => vr7_internal, 
			q     => vr7_reg
    );

  r8 : reg_dw 
	  generic map (
		  DW => DW
    )
		port map (
		  clk   => clk,
			reset => reset,
			load  => vr8_en,
			d     => vr8_internal, 
			q     => vr8_reg
    );

  -- Concurrent assignments
	vr7 <= vr7_reg;
	vr8 <= vr8_reg;

end struct;
